package be.iminds.ilabt.jfed.highlevel.jobs.link_test;

import be.iminds.ilabt.jfed.highlevel.jobs.TestLinksJob;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Consumer;

/**
 * LinkTestLogger
 */
public class LinkTestLogger {
    private static final Logger LOG = LoggerFactory.getLogger(LinkTestLogger.class);
    private final List<LinkTestListener> listeners = new CopyOnWriteArrayList<>();
    @Nullable private final Thread.UncaughtExceptionHandler uncaughtExceptionHandler;
    private final AtomicInteger exceptionCount = new AtomicInteger(0);

    public LinkTestLogger() {
        this.uncaughtExceptionHandler = Thread.currentThread().getUncaughtExceptionHandler();
    }
    public LinkTestLogger(@Nullable Thread.UncaughtExceptionHandler uncaughtExceptionHandler) {
        this.uncaughtExceptionHandler = uncaughtExceptionHandler;
    }


    public void addLinkTestListener(@Nonnull LinkTestListener listener) {
        listeners.add(listener);
    }
    public void removeLinkTestListener(@Nonnull LinkTestListener listener) {
        listeners.remove(listener);
    }
    

    public void fireAllReports(@Nonnull List<TestLinksJob.LinkTestReport> reports,
                          @Nonnull TestLinksJob.LinkTestResult worstResult)  {
        executeSafeForAllListeners(l -> l.onAllReports(reports, worstResult));
    }
    public void fireReport(@Nonnull TestLinksJob.LinkTestReport report)  {
        executeSafeForAllListeners(l -> l.onReport(report));
    }


    private void executeSafeForAllListeners(Consumer<LinkTestListener> c) {
        for (LinkTestListener l : listeners) {
            try {
                c.accept(l);
            } catch (Exception e) {
                handleExceptionInListener(e);
            }
        }
    }

    /**
     * We only handle the first exception with an uncaughtExceptionhandler, if present.
     * Otherwise, the user might see too much similar exceptions.
     *
     * @param e the exception that needs handling
     */
    private void handleExceptionInListener(Exception e) {
        int count = exceptionCount.getAndIncrement();
        if (count == 0 && uncaughtExceptionHandler != null) {
            try {
                uncaughtExceptionHandler.uncaughtException(Thread.currentThread(), e);
            } catch (Exception e2) {
                LOG.error("Ignoring exception in uncaughtExceptionHandler", e2);
            }
        } else {
            LOG.error("Ignoring exception " + count + " in LinkTester listener", e);
        }
    }
}
